@@ -10,17 +10,25 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+# flake8: noqa: F405
+
+from keystoneauth1.identity.v3.application_credential import *  # noqa
 from keystoneauth1.identity.v3.base import *  # noqa
 from keystoneauth1.identity.v3.federation import *  # noqa
 from keystoneauth1.identity.v3.k2k import *  # noqa
+from keystoneauth1.identity.v3.multi_factor import *  # noqa
 from keystoneauth1.identity.v3.oidc import *  # noqa
 from keystoneauth1.identity.v3.password import *  # noqa
+from keystoneauth1.identity.v3.receipt import *  # noqa
 from keystoneauth1.identity.v3.token import *  # noqa
 from keystoneauth1.identity.v3.totp import *  # noqa
 from keystoneauth1.identity.v3.tokenless_auth import *  # noqa
 
 
-__all__ = ('Auth',
+__all__ = ('ApplicationCredential',
+           'ApplicationCredentialMethod',
+
+           'Auth',
            'AuthConstructor',
            'AuthMethod',
            'BaseAuth',
@@ -43,4 +51,8 @@ __all__ = ('Auth',
            'TOTPMethod',
            'TOTP',
 
-           'TokenlessAuth')
+           'TokenlessAuth',
+
+           'ReceiptMethod',
+
+           'MultiFactor', )
