@@ -15,9 +15,9 @@ import itertools
 import json
 import logging
 import sys
+from unittest import mock
 import uuid
 
-import mock
 from oslo_utils import encodeutils
 import requests
 import requests.auth
@@ -164,6 +164,17 @@ class SessionTests(utils.TestCase):
         self.assertEqual(headers['OpenStack-API-Version'], 'volume 2.30')
         self.assertEqual(len(headers.keys()), 1)
 
+        # shared file system service-type - shared-file-system microversion
+        # (with service type aliases)
+        for service_type in ['sharev2', 'shared-file-system']:
+            headers = {}
+            client_session.Session._set_microversion_headers(
+                headers, (2, 30), None, {'service_type': service_type})
+            self.assertEqual(headers['X-OpenStack-Manila-API-Version'], '2.30')
+            self.assertEqual(headers['OpenStack-API-Version'],
+                             'shared-file-system 2.30')
+            self.assertEqual(len(headers.keys()), 2)
+
         # Headers already exist - no change
         headers = {
             'OpenStack-API-Version': 'compute 2.30',
@@ -421,24 +432,66 @@ class SessionTests(utils.TestCase):
         self.assertIn('--cacert', self.logger.output)
         self.assertIn(path_to_certs, self.logger.output)
 
-    def test_connect_retries(self):
+    def _connect_retries_check(self, session, expected_retries=0,
+                               call_args=None):
+        call_args = call_args or {}
+
         self.stub_url('GET', exc=requests.exceptions.Timeout())
 
-        session = client_session.Session()
-        retries = 3
+        call_args['url'] = self.TEST_URL
 
         with mock.patch('time.sleep') as m:
             self.assertRaises(exceptions.ConnectTimeout,
                               session.get,
-                              self.TEST_URL, connect_retries=retries)
+                              **call_args)
 
-            self.assertEqual(retries, m.call_count)
+            self.assertEqual(expected_retries, m.call_count)
             # 3 retries finishing with 2.0 means 0.5, 1.0 and 2.0
             m.assert_called_with(2.0)
 
         # we count retries so there will be one initial request + 3 retries
         self.assertThat(self.requests_mock.request_history,
-                        matchers.HasLength(retries + 1))
+                        matchers.HasLength(expected_retries + 1))
+
+    def test_session_connect_retries(self):
+        retries = 3
+        session = client_session.Session(connect_retries=retries)
+        self._connect_retries_check(session=session, expected_retries=retries)
+
+    def test_call_args_connect_retries_session_init(self):
+        session = client_session.Session()
+        retries = 3
+        call_args = {'connect_retries': retries}
+        self._connect_retries_check(session=session,
+                                    expected_retries=retries,
+                                    call_args=call_args)
+
+    def test_call_args_connect_retries_overrides_session_retries(self):
+        session_retries = 6
+        call_arg_retries = 3
+        call_args = {'connect_retries': call_arg_retries}
+        session = client_session.Session(connect_retries=session_retries)
+        self._connect_retries_check(session=session,
+                                    expected_retries=call_arg_retries,
+                                    call_args=call_args)
+
+    def test_override_session_connect_retries_for_request(self):
+        session_retries = 1
+        session = client_session.Session(connect_retries=session_retries)
+
+        self.stub_url('GET', exc=requests.exceptions.Timeout())
+        call_args = {'connect_retries': 0}
+
+        with mock.patch('time.sleep') as m:
+            self.assertRaises(
+                exceptions.ConnectTimeout,
+                session.request,
+                self.TEST_URL,
+                'GET',
+                **call_args
+            )
+
+            self.assertEqual(0, m.call_count)
 
     def test_connect_retries_interval_limit(self):
         self.stub_url('GET', exc=requests.exceptions.Timeout())
@@ -458,6 +511,25 @@ class SessionTests(utils.TestCase):
         self.assertThat(self.requests_mock.request_history,
                         matchers.HasLength(retries + 1))
 
+    def test_connect_retries_fixed_delay(self):
+        self.stub_url('GET', exc=requests.exceptions.Timeout())
+
+        session = client_session.Session()
+        retries = 3
+
+        with mock.patch('time.sleep') as m:
+            self.assertRaises(exceptions.ConnectTimeout,
+                              session.get,
+                              self.TEST_URL, connect_retries=retries,
+                              connect_retry_delay=0.5)
+
+            self.assertEqual(retries, m.call_count)
+            m.assert_has_calls([mock.call(0.5)] * retries)
+
+        # we count retries so there will be one initial request + 3 retries
+        self.assertThat(self.requests_mock.request_history,
+                        matchers.HasLength(retries + 1))
+
     def test_http_503_retries(self):
         self.stub_url('GET', status_code=503)
 
@@ -514,6 +586,26 @@ class SessionTests(utils.TestCase):
         self.assertThat(self.requests_mock.request_history,
                         matchers.HasLength(1))
 
+    def test_http_status_retries_fixed_delay(self):
+        self.stub_url('GET', status_code=409)
+
+        session = client_session.Session()
+        retries = 3
+
+        with mock.patch('time.sleep') as m:
+            self.assertRaises(exceptions.Conflict,
+                              session.get,
+                              self.TEST_URL, status_code_retries=retries,
+                              status_code_retry_delay=0.5,
+                              retriable_status_codes=[503, 409])
+
+            self.assertEqual(retries, m.call_count)
+            m.assert_has_calls([mock.call(0.5)] * retries)
+
+        # we count retries so there will be one initial request + 3 retries
+        self.assertThat(self.requests_mock.request_history,
+                        matchers.HasLength(retries + 1))
+
     def test_http_status_retries_inverval_limit(self):
         self.stub_url('GET', status_code=409)
 
@@ -891,6 +983,14 @@ class SessionAuthTests(utils.TestCase):
 
         self.assertRequestHeaderEqual('X-Auth-Token', None)
 
+    def test_object_delete(self):
+        auth = AuthPlugin()
+        sess = client_session.Session(auth=auth)
+        mock_close = mock.Mock()
+        sess._session.close = mock_close
+        del sess
+        self.assertEqual(1, mock_close.call_count)
+
     def test_service_type_urls(self):
         service_type = 'compute'
         interface = 'public'
@@ -1160,7 +1260,6 @@ class SessionAuthTests(utils.TestCase):
         self.assertIn(list(response.keys())[0], output)
         self.assertIn(list(response.values())[0], output)
 
-        self.assertNotIn(self.TEST_URL, self.logger.output)
         self.assertNotIn(list(response.keys())[0], self.logger.output)
         self.assertNotIn(list(response.values())[0], self.logger.output)
 
@@ -1511,7 +1610,6 @@ class AdapterTest(utils.TestCase):
         self.assertIn(list(response.keys())[0], output)
         self.assertIn(list(response.values())[0], output)
 
-        self.assertNotIn(self.TEST_URL, self.logger.output)
         self.assertNotIn(list(response.keys())[0], self.logger.output)
         self.assertNotIn(list(response.values())[0], self.logger.output)
 
@@ -1735,7 +1833,7 @@ class AdapterTest(utils.TestCase):
             with mock.patch.object(sess, 'request') as m:
                 adapter.Adapter(sess, **adap_kwargs).get(url, **get_kwargs)
                 m.assert_called_once_with(url, 'GET', endpoint_filter={},
-                                          rate_semaphore=mock.ANY,
+                                          headers={}, rate_semaphore=mock.ANY,
                                           **exp_kwargs)
 
         # No default_microversion in Adapter, no microversion in get()
@@ -1759,7 +1857,7 @@ class AdapterTest(utils.TestCase):
             with mock.patch.object(sess, 'request') as m:
                 adapter.Adapter(sess, **adap_kwargs).get(url, **get_kwargs)
                 m.assert_called_once_with(url, 'GET', endpoint_filter={},
-                                          rate_semaphore=mock.ANY,
+                                          headers={}, rate_semaphore=mock.ANY,
                                           **exp_kwargs)
 
         # No raise_exc in Adapter or get()
